//********************************************************************
//*** POSIX/timer.cc
//*** Copyright (c) 2003 by Markus Winand <mws@fatalmind.com>
//*** $Id: timer.cc,v 1.4 2003/10/26 19:50:38 mws Exp $
//********************************************************************

#ifndef POSIX_TIMER_HH
#include "timer.hh"
#endif

#ifndef INCLUDED_ERRNO_H
#include "Exceptions/Errno.hh"
#endif

#include <cmath>
#include <iomanip>
#include <iostream>


namespace fatalmind {

using fatalmind::POSIX::timespec;

template<class TMTS, class TM>
timer<TMTS, TM>::timer(timespec<TMTS>& tm, bool startnow)
: _start()
, _tm(tm)
, _running(startnow)
{ 
	if (_running) {
		start();
	}
}

template<class TMTS, class TM>
timer<TMTS, TM>::~timer()
{
	if (_running) {
		try {
			stop();
		} catch (...) {
			// exceptions in destructors are bad.
		}
	} 
}

template<class TMTS, class TM>
void timer<TMTS, TM>::start() {
	if( clock_gettime( CLOCK_REALTIME, &_start) == -1 ) {
		Errno::ThrowOnError(errno);
	}
	_running = true;
}

template<class TMTS, class TM>
void timer<TMTS, TM>::stop() {
	timespec<TM> stop;
	if( clock_gettime( CLOCK_REALTIME, &stop) == -1 ) {
		Errno::ThrowOnError(errno);
	}
	stop-=_start;
	
	if (stop < 0) {
		stop+=86400;
	}
	_tm+= stop;
	_running = false;
}

/*
std::ostream& operator<<(std::ostream& os, const timer& ts)
{
	char fill = os.fill();
	return os << ts.tv_sec 
			// TODO: locale
			<< '.' << std::setw(9) << std::setfill('0') << ts.tv_nsec 
			<< std::setfill(fill);
}
*/

} // namespace fatalmind

