//********************************************************************
//*** ResourcePool/SQL/NULL.t.cc
//*** Copyright (c) 2003-2004 by Markus Winand <mws@fatalmind.com>
//*** $Id: NULL.t.cc,v 1.8 2004/11/17 14:39:15 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

std::auto_ptr<RP> _rp;

void test_init()
{
	if (_rp.get() == 0) {
		_rp.reset(new RP(getFactory()));
	}
}

void test_NULL_init()
{
	test_init();
	drop(*_rp);
	init(*_rp);
}

void test_NULL_insert()
{
	RP::Command::SQLExecute ins("insert into hirsch (id, ukn) values (1000, null)");
	_rp->execute(ins);
	RP::Command::SQLExecute ins2("insert into hirsch (id, ukn) values (?, ?)");
	std::string nul("notnull");
	ins2.bind(0, 1001);
	ins2.bind(1, nul, true);
	_rp->execute(ins2);
	ins2.bind(0, 1002);
	ins2.bind(1, nul);
	_rp->execute(ins2);
}

void test_NULL_select()
{
	string sql("select id, ukn from hirsch where id = ?");
	RP::Command::SQLSelectRow cmd(sql);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);
	cmd.bind(0, 1000);
	_rp->execute(cmd);
	assertTrue(cmd.isnull(1));
	assertEqualT(int, col1, 1000);

	cmd.bind(0, 1001);
	_rp->execute(cmd);
	assertTrue(cmd.isnull(1));
	assertEqualT(int, col1, 1001);

	cmd.bind(0, 1002);
	_rp->execute(cmd);
	assertEqualT(int, col1, 1002);
	assertFalse(cmd.isnull(1));
}

void test_NULL_cleanup() {
	drop(*_rp);
}
