//********************************************************************
//*** ResourcePool/TaskRunner/TaskRunnerPool.cc
//*** Copyright (c) 2003 by Markus Winand <mws@fatalmind.com>
//*** $Id: TaskRunnerPool.cc,v 1.3 2004/08/23 17:39:53 mws Exp $
//********************************************************************

#include "ResourcePool/TaskRunner/TaskRunnerPool.hh"

// we instantiate the respective ResourcePool here
#include "ResourcePool/ResourcePool.cc"
#include "ResourcePool/Execute.cc"
#include "ResourcePool/Command.cc"

namespace fatalmind {

/*
 * 
 * CleanUpTask
 *
 */

namespace TaskRunnerPoolInternals {

class ReleaseTaskRunner {
	public:
		ReleaseTaskRunner(TaskRunnerPool& trp
			, TaskRunner& tr)
		: _trp(trp)
		, _tr(tr)
		{
		}
		
		~ReleaseTaskRunner()
		{
			_trp.free(_tr);
		}
	
	private:
		TaskRunnerPool& _trp;
		TaskRunner&     _tr;
};
	
class CleanUpTask: public fatalmind::Task {
	public:
		CleanUpTask(TaskRunnerPool& trp, TaskRunner& tr, Task& tsk)
		: _trp(trp)
		, _tr(tr)
		, _tsk(tsk)
		{
		}
		
		virtual void run() {
			ReleaseTaskRunner(_trp, _tr);	
			_tsk.run();
			//_tr.run(_tsk);
		}	

	private:
		TaskRunnerPool& _trp;
		TaskRunner&     _tr;
		Task&           _tsk;
};

} // namespace TaskRunnerPoolInternals

/*
 *
 * TaskRunnerPool
 *
 */

TaskRunnerPool::TaskRunnerPool(const ResourcePoolOptions& rpo)
: _factory()
, _pool(_factory, rpo)
{
}

TaskRunnerPool::~TaskRunnerPool()
{
}

void TaskRunnerPool::runDetached(Task& tsk) 
{
	TaskRunner& tr = _pool.get();
	TaskRunnerPoolInternals::CleanUpTask* cut 
		= new TaskRunnerPoolInternals::CleanUpTask(*this, tr, tsk);
	tr.runDetached(*cut);
}

TaskRunner::ID TaskRunnerPool::run(Task& tsk) 
{
	TaskRunner& tr = _pool.get();
	TaskRunnerPoolInternals::CleanUpTask* cut 
		= new TaskRunnerPoolInternals::CleanUpTask(*this, tr, tsk);
	return tr.run(*cut);
}

void TaskRunnerPool::free(TaskRunner& tr)
{
	_pool.free(tr);
}

} // namespace fatalmind
