//********************************************************************
//*** ResourcePool/mysql/MysqlException.cc
//*** Copyright (c) 2003 by Markus Winand <mws@fatalmind.com>
//*** $Id: MysqlException.cc,v 1.5 2003/10/01 20:01:12 mws Exp $
//********************************************************************

#ifndef RESOURCEPOOL_MYSQL_MYSQLEXCEPTION_HH
#include "MysqlException.hh"

#include <mysql/errmsg.h>

namespace fatalmind {

MysqlException::MysqlException(
	  const std::string& message
	, const int errorcode
	, const std::string& sql
):Exception(message)
, SQLException(message, mapMYSQLtoXOPEN(errorcode), sql, true)
, _errorcode(errorcode)
{
}

void MysqlException::throwit(
	  const int rv
	, const std::string& msg
	, const int errorcode
	, const std::string& sql
) {
	if (rv == CR_SERVER_LOST || rv == CR_SERVER_GONE_ERROR) {
		throw NFOMysqlException(msg, errorcode, sql);
	} else {
		std::string state(mapMYSQLtoXOPEN(errorcode));
		if (!state.compare(0,2, "08")) {
			throw MysqlException(msg, errorcode, sql);
		} else {
			throw NFOMysqlException(msg, errorcode, sql);
		}
	}
}

const char* MysqlException::mapMYSQLtoXOPEN(const int errorcode)
{
	switch(errorcode) {
		case 1058:
		case 1064:
		case 1136:
		case 1149: return "07000";
		case 1172: return "22000";
		case 1048:
		case 1062: return "23000";
		case 98: return "07001";
		case 99: return "02000";
	};
	return "08000"; //connection error
}

NFOMysqlException::NFOMysqlException(
	  const std::string& message
	, const int errorcode
	, const std::string& sql
):SQLException(message, mapMYSQLtoXOPEN(errorcode), sql, false)
, MysqlException(message, errorcode, sql)
, NoFailoverException(message)
{
}

} // namespace fatalmind
#endif


