//********************************************************************
//*** ResourcePool/mysql/Resource.cc
//*** Copyright (c) 2003 by Markus Winand <mws@fatalmind.com>
//*** $Id: Resource.cc,v 1.4 2003/09/14 08:25:53 mws Exp $
//********************************************************************

#ifndef RESOURCEPOOL_MYSQL_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_CREATEXCEPTION_HH
#include "ResourcePool/CreateException.hh"
#endif

#include <mysql/mysql.h>

#include <string>
#include <sstream>

namespace fatalmind {

mysqlResource::mysqlResource(const std::string& host
            , const std::string& user
            , const std::string& pass
            , const std::string& db
            , unsigned int port 
            , const std::string& sock 
)
: _dbh(0)
{
	if ((_dbh = mysql_init(_dbh)) == NULL) {
		throw CreateException(
			"The mysql_init function returned NULL"
		);
	}

	if (mysql_real_connect(_dbh
			, host.c_str()
			, user.c_str()
			, pass.c_str()
			, db.c_str()
			, port
			, NULL
			, 0) == NULL
	) {
		std::ostringstream ss;
		ss << "Can't connect to database: " << mysql_error(_dbh);
		throw CreateException(ss.str());
	}
}

bool mysqlResource::precheck() 
{
	if (mysql_ping(_dbh)) {
		return false;
	}
	return true;
}

bool mysqlResource::postcheck() 
{
	return true;
}

void mysqlResource::close()
{
	if (_dbh != NULL) {
		mysql_close(_dbh);
		_dbh = NULL;
	}
}

void mysqlResource::fail_close()
{
	if (_dbh != NULL) {
		mysql_close(_dbh);
		_dbh = NULL;
	}
}

MYSQL& mysqlResource::get_plain_resource()
{
	return *_dbh;
}

mysqlResource::~mysqlResource()
{
	close();
}

} // namespace fatalmind
