//********************************************************************
//*** ResourcePool/mysql/SQLSelect.hh
//*** Copyright (c) 2003 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelect.hh,v 1.9 2003/10/27 21:27:38 mws Exp $
//********************************************************************

#ifndef RESOURCEPOOL_MYSQL_SQLSELECT_HH
#define RESOURCEPOOL_MYSQL_SQLSELECT_HH

#ifndef RESOURCEPOOL_COMMAND_HH
#include "ResourcePool/Command.hh"
#endif

#ifndef RESOURCEPOOL_MYSQL_MYSQLFWD_HH
#include "mysqlFwd.hh"
#endif

#ifndef RESOURCEPOOL_MYSQL_SQLTYPE_HH
#include "SQLType.hh"
#endif

#ifndef INCLUDED_GC_PTR_HH
#include "mm/gc_ptr.hh"
#endif

#ifndef RESOURCEPOOL_SQL_BIND_HH
#include "ResourcePool/SQL/Bind.hh"
#endif

#ifndef RESOURCEPOOL_SQL_BINDOUT_HH
#include "ResourcePool/SQL/BindOut.hh"
#endif

#include <string>
#include <vector>

namespace fatalmind {
namespace mysql {
namespace internal {
class registerFinish;
}

class SQLSelect
: public Command<MYSQL> 
, public SQL::Bind<SQLType, SQLTypeImpl>
, public SQL::BindOut<SQLRefType, SQLRefTypeImpl>
{
	public:
		SQLSelect(const std::string& SQL);
		virtual ~SQLSelect();
		virtual void execute(MYSQL&);
		virtual bool process() = 0;

	protected:
		std::string _SQL;
	private:
		virtual bool fetch();
		void finish();
		void close_rs();
	
		MYSQL_RES* _rs;
		unsigned int _num_fields;
	friend
		class internal::registerFinish;
};

namespace internal {
class registerFinish {
	public:
		registerFinish(SQLSelect& select)
		: _select(select)
		{
		}
		~registerFinish()
		{
			_select.finish();
		}
	private:
		SQLSelect& _select;
};
} // namespace internal


} // namespace mysql
} // namespace fatalmind
#endif
