//********************************************************************
//*** ResourcePool/mysql/SQLType.cc
//*** Copyright (c) 2003 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLType.cc,v 1.4 2003/10/09 19:56:38 mws Exp $
//********************************************************************

#ifndef RESOURCEPOOL_MYSQL_SQLTYPE_CC
#define RESOURCEPOOL_MYSQL_SQLTYPE_CC

#ifndef RESOURCEPOOL_MYSQL_SQLTYPE_HH
#include "SQLType.hh"
#endif

#ifndef RESOURCEPOOL_MYSQL_MYSQLEXCEPTION_HH
#include "MysqlException.hh"
#endif

#include <sstream>
#include <limits>

namespace fatalmind {
namespace mysql {

template<typename T>
SQLTypeImpl<T>::SQLTypeImpl(const T& val) 
: _val(val)
{
}

template<typename T>
SQLTypeImpl<T>::~SQLTypeImpl() 
{
}

template<typename T>
bool SQLTypeImpl<T>::needsquote() const 
{
	return false;
}

template<typename T>
std::ostream& SQLTypeImpl<T>::opostream(std::ostream& os) const
{
	return os << _val;
}

/*
 * String specialization
 */

template<>
bool SQLTypeImpl<std::string>::needsquote() const
{
	return true;
}

/*
 * Instantiation
 */

template class SQLTypeImpl<short int>;
template class SQLTypeImpl<int>;
template class SQLTypeImpl<long int>;
template class SQLTypeImpl<unsigned short int>;
template class SQLTypeImpl<unsigned int>;
template class SQLTypeImpl<unsigned long int>;
template class SQLTypeImpl<float>;
template class SQLTypeImpl<double>;
template class SQLTypeImpl<std::string>;

/*
 *
 * SQLRefType
 *
 */

template<typename T>
SQLRefTypeImpl<T>::SQLRefTypeImpl(T& val)
: _val(val)
{
}

template<typename T>
SQLRefTypeImpl<T>::~SQLRefTypeImpl()
{
}

template<typename T>
void SQLRefTypeImpl<T>::assign(const char* chr, const int len) 
{
	_valptr = chr;
	if (!isnull()) {
		std::string str(chr, len);
		std::istringstream is(str);
		if (!(is >> _val)) {
			throw NFOMysqlException("value does not fit into specified type", 98, "UKN");
		}
		/* really strange, the above code, fails to fail for unsigned int and unsigned long
		 * but works for unsiged short
		 * maybe a compiler/library problem (problem happend with GCC 3.3.1)
		 * however, for now i will do the check manually */
		if (str[0] == '-' && ! std::numeric_limits<T>::is_signed) {
			throw NFOMysqlException("value does not fit into specified type", 98, "UKN");
		}
	}
}

template<>
void SQLRefTypeImpl<std::string>::assign(const char* chr, const int len)
{
	_valptr = chr;
	if (!isnull()) {
		_val.assign(chr, len);
	}
}

/*
template<typename T>
std::istream& SQLRefTypeImpl<T>::opistream(std::istream& is)
{
	return is >> _val;
}
*/

/*
 * Instantioation
 */

template class SQLRefTypeImpl<short int>;
template class SQLRefTypeImpl<int>;
template class SQLRefTypeImpl<long>;
template class SQLRefTypeImpl<unsigned short int>;
template class SQLRefTypeImpl<unsigned int>;
template class SQLRefTypeImpl<unsigned long>;
template class SQLRefTypeImpl<float>;
template class SQLRefTypeImpl<double>;
template class SQLRefTypeImpl<std::string>;

} // namespace mysql
} // namespace fatalmind
#endif
