//********************************************************************
//*** ResourcePool/mysql/SQLType.hh
//*** Copyright (c) 2003 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLType.hh,v 1.5 2003/09/24 20:39:27 mws Exp $
//********************************************************************

#ifndef RESOURCEPOOL_MYSQL_SQLTYPE_HH
#define RESOURCEPOOL_MYSQL_SQLTYPE_HH

#include <iostream>

namespace fatalmind {
namespace mysql {

class SQLType {
	public:
		SQLType() 
		: _nul(false)
		{
		}
		virtual ~SQLType() 
		{
		}
	
		virtual bool needsquote() const = 0;	
		virtual std::ostream& opostream(std::ostream&) const = 0;
		void setnull(bool nul = true) {
			_nul = nul;
		};
		bool isnull() const throw() {
			return _nul;
		}

	private:
		bool _nul;
};

template<typename T>
class SQLTypeImpl: public SQLType
{
	public:
		SQLTypeImpl<T>(const T& val);
		virtual ~SQLTypeImpl<T>();

		virtual bool needsquote() const;
		virtual std::ostream& opostream(std::ostream&) const;
		
	private:
		T _val;
};

class SQLRefType {
	public:
		virtual void assign(const char*, const int len) = 0;
		bool isnull() const throw() {
			return (_valptr == NULL);
		}
	protected:
		const void* _valptr;
};

template<typename T>
class SQLRefTypeImpl: public SQLRefType 
{
	public:
		SQLRefTypeImpl<T>(T& val);
		virtual ~SQLRefTypeImpl<T>();
		virtual void assign(const char*, const int len);
	private:
		T& _val;
};

inline std::ostream& operator<<(std::ostream& os, const SQLType& val)
{
	return val.opostream(os);
}

} // namespace mysql
} // namespace fatalmind
#endif
