//********************************************************************
//*** ResourcePool/oracle/Factory.hh
//*** Copyright (c) 2003-2004 by Markus Winand <mws@fatalmind.com>
//*** $Id: Factory.hh,v 1.12 2004/11/17 14:39:17 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_FACTORY_HH
#define RESOURCEPOOL_ORACLE_FACTORY_HH

#ifndef RESOURCEPOOL_ORACLE_RESOURCEFWD_HH
#include "ResourceFwd.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_OCIENVHANDLEFWD_HH
#include "OCIEnvHandleFwd.hh"
#endif

#ifndef INCLUDED_GC_PTR_HH
#include "mm/gc_ptr.hh"
#endif

#include <string>
#include <memory>

namespace fatalmind {

/**
 * A Factory for a oracle database connection (session).
 * Thats the place where you configure which database
 * user and password to use.
 */
template<class TM /* a Threading model */
, template<class> class SLMDC = oracle::SelectListMetaDataCache 
 /* Alternatively you could use oracle::SelectListMetaDataNopCache */
> class oracleFactory {
	public:
		/**
		 * \param service_name the TNS name of the database
		 * \param user the user name
		 * \param pass the users password
		 */
		oracleFactory(const std::string& service_name
			, const std::string& user
			, const std::string& pass
		);
		oracleFactory(const oracleFactory&);
		~oracleFactory();

		oracleResource<TM>* create_resource() const;

	private:
		const std::string _service_name;
		const std::string _user;
		const std::string _pass;

		const unsigned int _mode;
		mutable std::auto_ptr<oracle::OCIEnvHandle> _env;
		gc_ptr<typename oracle::SelectListMetaDataCacheInterface<TM>::type, TM> _slmdc;
};

} // namespace fatalmind
#endif

