//********************************************************************
//*** ResourcePool/oracle/SQLExecute.cc
//*** Copyright (c) 2003-2004 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLExecute.cc,v 1.19 2004/11/17 14:39:17 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_SQLEXECUTE_HH
#include "SQLExecute.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_OCIHANDLE_HH
#include "OCIHandle.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLETYPE_HH
#include "oracleType.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#include "OracleExceptionFactory.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_COMMANDINTERFACE_CC
#include "CommandInterface.cc"
#endif

namespace fatalmind {
namespace oracle {

template<class TM>
SQLExecute<TM>::SQLExecute(const std::string& SQL)
: CommandCommon<TM>(SQL)
{
}


template<class TM>
CommandCommon<TM>* SQLExecute<TM>::clone() const 
{
	return new SQLExecute<TM>(*this);
};


template<class TM>
void SQLExecute<TM>::_execute(oracleHandlesWithCache<TM>& hand, StatementCache& sthcache) 
{
	sword ret;
	prepare(hand, sthcache, oracleStatement(_SQL));

	real_bind(hand);

	/* execute statement */
	if ((ret = OCIStmtExecute(hand.getSvcCtx()
		, _sth->get()
		, hand.getError()
		, (ub4) 1
		, (ub4) 0
		, (CONST OCISnapshot *) 0
		, (OCISnapshot *) 0
		, (ub4) OCI_DEFAULT | super::_ac_mode()))
	) {
		OracleExceptionFactory::throwit("OCIStmtExecute", hand, ret, _SQL);
	}
}

template class SQLExecute<DefaultThreadedModel >;

} // namespace oracle
} // namespace fatalmind

