//********************************************************************
//*** ResourcePool/oracle/SQLSelect.cc
//*** Copyright (c) 2003-2004 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelect.cc,v 1.23 2004/11/17 14:39:17 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_SQLSELECT_HH
#include "SQLSelect.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLETYPE_HH
#include "oracleType.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEREFTYPE_HH
#include "oracleRefType.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#include "OracleExceptionFactory.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_SELECTLISTMETADATA_HH
#include "SelectListMetaData.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_COMMANDINTERFACE_CC
#include "CommandInterface.cc"
#endif

namespace fatalmind {
namespace oracle {

template<class TM>
SQLSelect<TM>::SQLSelect(const std::string& SQL) 
: SQLSelectRow<TM>(SQL)
{
}

template<class TM>
SQLSelect<TM>::SQLSelect(const SQLSelect& rho) 
: SQLSelectRow<TM>(rho) 
{
}

template<class TM>
SQLSelect<TM>::~SQLSelect()
{
}

template<class TM>
void SQLSelect<TM>::_execute(oracleHandlesWithCache<TM>& hand, StatementCache& sthcache) 
{
	sword ret;
	prepare_buffer(hand, sthcache);

	real_bind(hand);
	preSelectBufferBind(hand);

	/* execute statement */
	if ((ret = OCIStmtExecute(hand.getSvcCtx()
		, _sth->get()
		, hand.getError()
		, (ub4) 0
		, (ub4) 0
		, (CONST OCISnapshot *) 0
		, (OCISnapshot *) 0
		, (ub4) OCI_DEFAULT | super::_ac_mode())
	) != OCI_SUCCESS) {
		OracleExceptionFactory::throwit("OCIStmtExecute", hand, ret, _SQL);
	}

	internal::registerFinish fin(hand, *this, *_sth, _SQL);
	postSelectBuffer(hand);
	while (fetch(hand, *_sth, _SQL) && process()) {
	}
}

template class SQLSelect<DefaultThreadedModel>;

} // namespace oracle
} // namespace fatalmind
