//********************************************************************
//*** ResourcePool/oracle/oracleRefType.cc
//*** Copyright (c) 2003-2004 by Markus Winand <mws@fatalmind.com>
//*** $Id: oracleRefType.cc,v 1.12 2004/11/17 14:39:17 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef INCLUDED_EXCEPTIONS_HH
#include "Exceptions/Exception.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEREFTYPE_HH
#include "oracleRefType.hh"
#endif

#include <string>
#include <memory>
#include <limits>

namespace fatalmind {
namespace oracle {

template<typename T>
oracleRefTypeImpl<T>::oracleRefTypeImpl(T& val)
: _val(val)
{
}

template<typename T>
oracleRefTypeImpl<T>::~oracleRefTypeImpl()
{
}

template<typename T>
void* oracleRefTypeImpl<T>::ptr() const {
	return &_val;
}

template<typename T>
sb4 oracleRefTypeImpl<T>::capacity() const {
	return sizeof(T);
}

template<typename T>
ub2 oracleRefTypeImpl<T>::type() const {
	return std::numeric_limits<T>::is_signed ? SQLT_INT : SQLT_UIN;
}

template<typename T>
void oracleRefTypeImpl<T>::start() {
}

template<typename T>
void oracleRefTypeImpl<T>::done(const ub2 size, void* ptr) {
}

/*
 * string specialization
 */

oracleRefTypeImpl<std::string>::oracleRefTypeImpl(std::string& val)
: _val(val)
, _cap(_val.capacity())
, _buf(new char[_cap])
{
}

oracleRefTypeImpl<std::string>::~oracleRefTypeImpl()
{
}

void* oracleRefTypeImpl<std::string>::ptr() const {
	return _buf.get();
}

sb4 oracleRefTypeImpl<std::string>::capacity() const {
	return _cap - 1;
}

ub2 oracleRefTypeImpl<std::string>::type() const {
	return SQLT_CHR;
}

void oracleRefTypeImpl<std::string>::start() {
	_val.clear();
}

void oracleRefTypeImpl<std::string>::done(const ub2 size, void* ptr) {
	_val.append((char*)ptr, size);
}

/*
 * speicalizations for built in types
 */

template<>
ub2 oracleRefTypeImpl<float>::type() const {
	return SQLT_FLT;
}

template<>
ub2 oracleRefTypeImpl<double>::type() const {
	return SQLT_FLT;
}

/*
 * Instantiation
 */

template class oracleRefTypeImpl<int>;
template class oracleRefTypeImpl<float>;
template class oracleRefTypeImpl<double>;
template class oracleRefTypeImpl<std::string>;

dvoid* oracleRefType::indptr() const {
	return (dvoid*)&_ind;
}

bool oracleRefType::isnull() const {
	return (_ind == -1);
}

/*
 * somthing never called
 */
oracleRefType::~oracleRefType() {
}

}
}
