//********************************************************************
//*** Thread/Synchronize.t.cc
//*** Copyright (c) 2003-2004 by Markus Winand <mws@fatalmind.com>
//*** $Id: Synchronize.t.cc,v 1.11 2004/11/17 14:39:22 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "Thread/Thread.hh"
#include "Exceptions/Errno.hh"
#include <string>
#include <stdio.h>

#include "Thread/Lock.hh"
#include "Thread/Synchronize.hh"

#include "t/main.hh"

using namespace fatalmind;

template<class Lock>
class MyThread: public Thread {
	public:
		MyThread(int num, Lock& pm, int& shared)
		: _private(0) 
		, _num(num)
		, _pm(pm)
		, _shared(shared)
		{};
		virtual ~MyThread() {};
	
		virtual void inc() {
			Synchronize<Lock> lk(_pm);
			int tmp = _shared;
			tmp++;
			_shared = tmp;
		}

		virtual void run() {
			for (_private = 0; _private < _num; ++_private) {
				inc();
				if (! _private % 1000) {
					Thread::yield();
				}
			}
		}

		int getPrivate() {
			return _private;
		}

	private:
		int _private;
		int _num;
		Lock &_pm;
		int& _shared;
};

template<class Lock>
class testit {
	public:
		testit()
		: _shared(0)
		, _lock()
		{}
		
		void run(int num) {
			MyThread<Lock> t(num, _lock, _shared);
			MyThread<Lock> t2(num, _lock, _shared);
			t2.start();
			assertNotEqual(_shared, num);
			t.start();
			t.join();
			t2.join();

			assertEqual(_shared, t.getPrivate() + t2.getPrivate());
		}
	private:
		int _shared;
		Lock _lock;
};

void TEST_Synchronized2() {
	{
		FastLock<> l1;
		FastLock<> l2;
		Synchronize2<FastLock<>, FastLock<> > sync(l1, l2);
	}
	{
		FastLock<> l1;
		Synchronize2<FastLock<>, FastLock<> > sync(l1, l1);
	}
}

void TEST_PERF_Synchronized_FastLock() {
	testit<FastLock<> > ti;
	ti.run(300000);
}

void TEST_PERF_Synchronized_RecursiveLock() {
	testit<RecursiveLock<> > ti;
	ti.run(300000);
}

void TEST_PERF_Synchronized_NopLock() {
	testit<NopLock<> > ti;
	try {
		ti.run(300000);
	} catch (...) {
		// just for performance masurements
	}
}

