//********************************************************************
//*** t/assert.cc
//*** Copyright (c) 2003 by Markus Winand <mws@fatalmind.com>
//*** $Id: assert.cc,v 1.6 2003/10/12 15:29:13 mws Exp $
//********************************************************************

#ifndef INCLUDED_T_ASSERT_CC
#define INCLUDED_T_ASSERT_CC

#ifndef INCLUDED_T_MAIN_HH
#include "main.hh"
#endif

#ifndef INCLUDED_EXCEPTIONS_HH
#include "Exceptions/Exception.hh"
#endif

#ifndef OPERATORS_EPSILON_EQ_HH
#include "operators/epsilon_eq.hh"
#endif

#include <sstream>
#include <iostream>
#include <iomanip>

using namespace std;
using fatalmind::operators::epsilon_eq;

template <typename T>
void _assertEqual(
	  typename fatalmind::arg_traits<T>::const_param is
	, typename fatalmind::arg_traits<T>::const_param expect
	, const char* file
	, int line)
{
	if (!(epsilon_eq(is, expect))) {
		ostringstream os;
		os << "assertEqual failed in " << file << ":"
			<< line << " (got " 
			<< std::fixed << std::setprecision(15) << is 
			<< " expected " << std::fixed << expect << ")";
		throw fatalmind::Exception(os.str());
	}
}

/*
void _assertEqual(const std::string& is, const char* expect, const char* file, int line) {
	_assertEqual(is, std::string(expect), file, line);
}
*/

template <class T>
void _assertLess(T actual, T limit, char* file, int line)
{
	if (!(actual < limit)) {
		ostringstream os;
		os << "assertLess failed in " << file << ":"
			<< line << " (" << actual << " is not less then " << limit << ")";
		throw fatalmind::Exception(os.str());
	}
}

template <class T>
void _assertNotLess(T actual, T limit, char* file, int line)
{
	if (actual < limit) {
		ostringstream os;
		os << "assertNotLess failed in " << file << ":"
			<< line << " (" << actual << " is less then " << limit << ")";
		throw fatalmind::Exception(os.str());
	}
}

template <class T>
void _assertMore(T actual, T limit, char* file, int line)
{
	if (!(actual > limit)) {
		ostringstream os;
		os << "assertMore failed in " << file << ":"
			<< line << " (" << actual << " is not more then " << limit << ")";
		throw fatalmind::Exception(os.str());
	}
}

template <class T>
void _assertNotMore(T actual, T limit, char* file, int line)
{
	if (actual > limit) {
		ostringstream os;
		os << "assertNotMore failed in " << file << ":"
			<< line << " (" << actual << " is more then " << limit << ")";
		throw fatalmind::Exception(os.str());
	}
}

template <class T>
void _assertNotEqual(T is, T expect, char* file, int line)
{
	if (is == expect) {
		ostringstream os;
		os << "assertNotEqual failed in " << file << ":"
			<< line << " (got " << is << " and " << expect << ")";
		throw fatalmind::Exception(os.str());
	}
}

#endif
